/*----- PROTECTED REGION ID(DataVampire.cpp) ENABLED START -----*/
/* clang-format on */
//=============================================================================
//
// file :        DataVampire.cpp
//
// description : C++ source for the DataVampire class and its commands.
//               The class is derived from Device. It represents the
//               CORBA servant object which will be accessed from the
//               network. All commands which can be executed on the
//               DataVampire are implemented in this file.
//
// project :     DataVampire
//
// This file is part of Tango device class.
//
// Tango is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Tango is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Tango.  If not, see <http://www.gnu.org/licenses/>.
//
//
//
//=============================================================================
//                This file is generated by POGO
//        (Program Obviously used to Generate tango Object)
//=============================================================================


#include "DataVampire.h"
#include "DataVampireClass.h"

#include "acqthread.h"


/* clang-format off */
/*----- PROTECTED REGION END -----*/	//	DataVampire.cpp

/**
 *  DataVampire class description:
 *    Get vectors of 1D or 2D data and calculate sum, mean, horpos, verpos, horsigma, versigma, numsamples
 */

//================================================================
//  The following table gives the correspondence
//  between command and method names.
//
//  Command name    |  Method name
//================================================================
//  State           |  Inherited (no method)
//  Status          |  Inherited (no method)
//  GetSum          |  get_sum
//  GetMean         |  get_mean
//  GetHorPos       |  get_hor_pos
//  GetVerPos       |  get_ver_pos
//  GetHorPosStd    |  get_hor_pos_std
//  GetVerPosStd    |  get_ver_pos_std
//  GetValidValues  |  get_valid_values
//  SetBackground   |  set_background
//================================================================

//================================================================
//  Attributes managed are:
//================================================================
//  MinThreshold           |  Tango::DevDouble	Scalar
//  MaxThreshold           |  Tango::DevDouble	Scalar
//  CommandString          |  Tango::DevString	Scalar
//  PollingPeriod          |  Tango::DevLong	Scalar
//  PollingSamples         |  Tango::DevLong	Scalar
//  Sum                    |  Tango::DevDouble	Scalar
//  Mean                   |  Tango::DevDouble	Scalar
//  HorPos                 |  Tango::DevDouble	Scalar
//  VerPos                 |  Tango::DevDouble	Scalar
//  HorPosStd              |  Tango::DevDouble	Scalar
//  ValidValues            |  Tango::DevLong	Scalar
//  EnableAcquisition      |  Tango::DevBoolean	Scalar
//  CacheSize              |  Tango::DevLong	Scalar
//  AutoBackgroundSamples  |  Tango::DevLong	Scalar
//  PostAcqRoi             |  Tango::DevBoolean	Scalar
//  Width                  |  Tango::DevLong	Scalar
//  Height                 |  Tango::DevLong	Scalar
//  VerPosStd              |  Tango::DevDouble	Scalar
//  EnablePolling          |  Tango::DevBoolean	Scalar
//  BackgroundSamples      |  Tango::DevLong	Scalar
//  RoiParam               |  Tango::DevLong	Spectrum  ( max = 4)
//  Spectrum               |  Tango::DevDouble	Spectrum  ( max = 1000000)
//  HorBackground          |  Tango::DevDouble	Spectrum  ( max = 1000000)
//  VerBackground          |  Tango::DevDouble	Spectrum  ( max = 1000000)
//  Image                  |  Tango::DevDouble	Image  ( max = 5000 x 5000)
//================================================================

namespace DataVampire_ns
{
/*----- PROTECTED REGION ID(DataVampire::namespace_starting) ENABLED START -----*/
/* clang-format on */
//	static initializations
/* clang-format off */
/*----- PROTECTED REGION END -----*/	//	DataVampire::namespace_starting

//--------------------------------------------------------
/**
 *	Method      : DataVampire::DataVampire()
 *	Description : Constructors for a Tango device
 *                implementing the classDataVampire
 */
//--------------------------------------------------------
DataVampire::DataVampire(Tango::DeviceClass *cl, string &s)
 : TANGO_BASE_CLASS(cl, s.c_str())
{
	/*----- PROTECTED REGION ID(DataVampire::constructor_1) ENABLED START -----*/
	/* clang-format on */
	init_device();
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	DataVampire::constructor_1
}
//--------------------------------------------------------
DataVampire::DataVampire(Tango::DeviceClass *cl, const char *s)
 : TANGO_BASE_CLASS(cl, s)
{
	/*----- PROTECTED REGION ID(DataVampire::constructor_2) ENABLED START -----*/
	/* clang-format on */
	init_device();
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	DataVampire::constructor_2
}
//--------------------------------------------------------
DataVampire::DataVampire(Tango::DeviceClass *cl, const char *s, const char *d)
 : TANGO_BASE_CLASS(cl, s, d)
{
	/*----- PROTECTED REGION ID(DataVampire::constructor_3) ENABLED START -----*/
	/* clang-format on */
	init_device();
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	DataVampire::constructor_3
}

//--------------------------------------------------------
/**
 *	Method      : DataVampire::delete_device()
 *	Description : will be called at device destruction or at init command
 */
//--------------------------------------------------------
void DataVampire::delete_device()
{
	DEBUG_STREAM << "DataVampire::delete_device() " << device_name << endl;
	/*----- PROTECTED REGION ID(DataVampire::delete_device) ENABLED START -----*/
	/* clang-format on */
	//	Delete device allocated objects
	/* clang-format off */
	abort_flag = true;
	if (acqloop) {
		acqloop->join(NULL);
		acqloop = 0;
	}	
	
	if (attr_CommandString_read) {
		delete [] *attr_CommandString_read;
		attr_CommandString_read = 0;
	}

	if (device)
		delete device;

	if (bn_device)
		delete bn_device;

	delete mutex;

	/*----- PROTECTED REGION END -----*/	//	DataVampire::delete_device
	delete[] attr_MinThreshold_read;
	delete[] attr_MaxThreshold_read;
	delete[] attr_CommandString_read;
	delete[] attr_PollingPeriod_read;
	delete[] attr_PollingSamples_read;
	delete[] attr_Sum_read;
	delete[] attr_Mean_read;
	delete[] attr_HorPos_read;
	delete[] attr_VerPos_read;
	delete[] attr_HorPosStd_read;
	delete[] attr_ValidValues_read;
	delete[] attr_EnableAcquisition_read;
	delete[] attr_CacheSize_read;
	delete[] attr_AutoBackgroundSamples_read;
	delete[] attr_PostAcqRoi_read;
	delete[] attr_Width_read;
	delete[] attr_Height_read;
	delete[] attr_VerPosStd_read;
	delete[] attr_EnablePolling_read;
	delete[] attr_BackgroundSamples_read;
	delete[] attr_RoiParam_read;
	delete[] attr_Spectrum_read;
	delete[] attr_HorBackground_read;
	delete[] attr_VerBackground_read;
	delete[] attr_Image_read;
}

//--------------------------------------------------------
/**
 *	Method      : DataVampire::init_device()
 *	Description : will be called at device initialization.
 */
//--------------------------------------------------------
void DataVampire::init_device()
{
	DEBUG_STREAM << "DataVampire::init_device() create device " << device_name << endl;
	/*----- PROTECTED REGION ID(DataVampire::init_device_before) ENABLED START -----*/
	/* clang-format on */
	//	Initialization before get_device_property() call
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	DataVampire::init_device_before
	

	//	Get the device properties from database
	get_device_property();
	
	attr_MinThreshold_read = new Tango::DevDouble[1];
	attr_MaxThreshold_read = new Tango::DevDouble[1];
	attr_CommandString_read = new Tango::DevString[1];
	attr_PollingPeriod_read = new Tango::DevLong[1];
	attr_PollingSamples_read = new Tango::DevLong[1];
	attr_Sum_read = new Tango::DevDouble[1];
	attr_Mean_read = new Tango::DevDouble[1];
	attr_HorPos_read = new Tango::DevDouble[1];
	attr_VerPos_read = new Tango::DevDouble[1];
	attr_HorPosStd_read = new Tango::DevDouble[1];
	attr_ValidValues_read = new Tango::DevLong[1];
	attr_EnableAcquisition_read = new Tango::DevBoolean[1];
	attr_CacheSize_read = new Tango::DevLong[1];
	attr_AutoBackgroundSamples_read = new Tango::DevLong[1];
	attr_PostAcqRoi_read = new Tango::DevBoolean[1];
	attr_Width_read = new Tango::DevLong[1];
	attr_Height_read = new Tango::DevLong[1];
	attr_VerPosStd_read = new Tango::DevDouble[1];
	attr_EnablePolling_read = new Tango::DevBoolean[1];
	attr_BackgroundSamples_read = new Tango::DevLong[1];
	attr_RoiParam_read = new Tango::DevLong[4];
	attr_Spectrum_read = new Tango::DevDouble[1000000];
	attr_HorBackground_read = new Tango::DevDouble[1000000];
	attr_VerBackground_read = new Tango::DevDouble[1000000];
	attr_Image_read = new Tango::DevDouble[5000*5000];
	/*----- PROTECTED REGION ID(DataVampire::init_device) ENABLED START -----*/
	/* clang-format on */
	//	Initialize device
	/* clang-format off */

	*attr_CommandString_read = new char[DATAVAMPIRE_MAX_STR_LEN];

	*attr_AutoBackgroundSamples_read = 0;
	*attr_MinThreshold_read = -99999;
	*attr_MaxThreshold_read = 99999;
	*attr_CacheSize_read = 100;
	*attr_PostAcqRoi_read = false;
	*attr_EnableAcquisition_read = true;
	*attr_PollingSamples_read = 10;
	*attr_PollingPeriod_read = 500;
	*attr_EnablePolling_read = false;
	*attr_BackgroundSamples_read = 0;

	num_rows_image = 0;
	num_cols_image = 0;
	num_cols_spectrum = 0;

	roi_dim = 0;
	for (int i = 0; i < 4; i++) {
		attr_RoiParam_read[i] = roiParam[i];
		if (attr_RoiParam_read[i] >= 0)
			roi_dim++;
	}

	std::vector<std::string> results;
	std::string new_configuration(commandString);
	string_explode(new_configuration, "/", &results);

	command_name = results[results.size()-1];
	size_t pos = new_configuration.find(command_name);
	if (pos != std::string::npos) { // if str2 found in str1
        	new_configuration.erase(pos-1, command_name.length()+1); // erase str2 from str1
    	}	
	c_device_name = new_configuration;

	INFO_STREAM << "Device name: " << c_device_name << endl;
	INFO_STREAM << "Command name: " << command_name << endl;


	try {
		device = 0;
		device = new Tango::DeviceProxy(c_device_name);

		Tango::CommandInfo cmd_info = device->command_query(command_name);
		cmd_in_type = cmd_info.in_type;
		cmd_out_type = cmd_info.out_type;
		snprintf(*attr_CommandString_read, DATAVAMPIRE_MAX_STR_LEN, "%s/%s", c_device_name.c_str(),command_name.c_str());
		set_state(Tango::ON);
		set_status("Connected to device");
	}	
	catch (Tango::DevFailed &e) {
		INFO_STREAM << e.errors[0].desc << ", " << c_device_name << " " << command_name << endl;
		INFO_STREAM << e.errors[0].reason << ", " << c_device_name << " " << command_name << endl;
		set_state(Tango::FAULT);
		set_status("Failed to execute device command, check CommandString property and restart device");			
	}	
	catch(...) {
		set_state(Tango::FAULT);
		set_status("Failed to execute device command, check CommandString property and restart device");
		INFO_STREAM << "Generic error, failed to execute command_query on device " << c_device_name << " " << command_name << endl;
	}	


	std::vector<std::string> results_a;
	std::string bn_attr(bunchNumberString);
	string_explode(bn_attr, "/", &results_a);	
	bn_attr_name = results_a[results_a.size()-1];
	pos = bn_attr.find(bn_attr_name);
	if (pos != std::string::npos) { // if str2 found in str1
        	bn_attr.erase(pos-1, bn_attr_name.length()+1); // erase str2 from str1
    	}	
	bn_device_name = bn_attr;
	
	INFO_STREAM << "BN device name: " << bn_device_name << endl;
	INFO_STREAM << "BN attr name: " << bn_attr_name << endl;
			
	if (bn_attr.size() > 0) {
		try {
			bn_device = new Tango::DeviceProxy(bn_device_name);
			INFO_STREAM << bn_device_name << endl;
			INFO_STREAM << bn_attr_name << endl;
		}	
		catch(...) {
			set_state(Tango::FAULT);
			set_status("Failed to connect to BunchNumber device");	
		}
	}

	for (int i = 0; i < DATAVAMPIRE_MAX_SIZE; i++) {
		acq_array[i].sum = 0;
		acq_array[i].mean =  0;
		acq_array[i].horpos = 0;
		acq_array[i].verpos = 0;		
		acq_array[i].horposstd = 0;
		acq_array[i].verposstd = 0;	
		acq_array[i].validvalues = 0;
		acq_array[i].bn = 0;		
	}	
	array_idx = 0;
		

	try {
		last_bn = 0;
		bn_device->read_attribute(bn_attr_name) >> last_bn;
	}
	catch(...){}

	try {
		frequency = 0;
		Tango::DevLong bunch_number_pre, bunch_number_post;
		bn_device->read_attribute(bn_attr_name) >> bunch_number_pre;
		sleep(1);
		bn_device->read_attribute(bn_attr_name) >> bunch_number_post;
		frequency = bunch_number_post - bunch_number_pre;
	}
	catch(...){}		
	
	if (frequency < 25)
		frequency = 10;
	else
		frequency = 50;	

	mutex = new omni_mutex();
	
	acqloop = 0;
	acqloop = new acqthread(this);
	acqloop->start_undetached();
	

	/*----- PROTECTED REGION END -----*/	//	DataVampire::init_device
}

//--------------------------------------------------------
/**
 *	Method      : DataVampire::get_device_property()
 *	Description : Read database to initialize property data members.
 */
//--------------------------------------------------------
void DataVampire::get_device_property()
{
	/*----- PROTECTED REGION ID(DataVampire::get_device_property_before) ENABLED START -----*/
	/* clang-format on */
	//	Initialize property data members
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	DataVampire::get_device_property_before


	//	Read device properties from database.
	Tango::DbData	dev_prop;
	dev_prop.push_back(Tango::DbDatum("HorCalib"));
	dev_prop.push_back(Tango::DbDatum("VerCalib"));
	dev_prop.push_back(Tango::DbDatum("RoiParam"));
	dev_prop.push_back(Tango::DbDatum("CommandString"));
	dev_prop.push_back(Tango::DbDatum("BunchNumberString"));

	//	is there at least one property to be read ?
	if (dev_prop.size()>0)
	{
		//	Call database and extract values
		if (Tango::Util::instance()->_UseDb==true)
			get_db_device()->get_property(dev_prop);
	
		//	get instance on DataVampireClass to get class property
		Tango::DbDatum	def_prop, cl_prop;
		DataVampireClass	*ds_class =
			(static_cast<DataVampireClass *>(get_device_class()));
		int	i = -1;

		//	Try to initialize HorCalib from class property
		cl_prop = ds_class->get_class_property(dev_prop[++i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  horCalib;
		else {
			//	Try to initialize HorCalib from default device value
			def_prop = ds_class->get_default_device_property(dev_prop[i].name);
			if (def_prop.is_empty()==false)	def_prop  >>  horCalib;
		}
		//	And try to extract HorCalib value from database
		if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  horCalib;

		//	Try to initialize VerCalib from class property
		cl_prop = ds_class->get_class_property(dev_prop[++i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  verCalib;
		else {
			//	Try to initialize VerCalib from default device value
			def_prop = ds_class->get_default_device_property(dev_prop[i].name);
			if (def_prop.is_empty()==false)	def_prop  >>  verCalib;
		}
		//	And try to extract VerCalib value from database
		if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  verCalib;

		//	Try to initialize RoiParam from class property
		cl_prop = ds_class->get_class_property(dev_prop[++i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  roiParam;
		else {
			//	Try to initialize RoiParam from default device value
			def_prop = ds_class->get_default_device_property(dev_prop[i].name);
			if (def_prop.is_empty()==false)	def_prop  >>  roiParam;
		}
		//	And try to extract RoiParam value from database
		if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  roiParam;

		//	Try to initialize CommandString from class property
		cl_prop = ds_class->get_class_property(dev_prop[++i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  commandString;
		else {
			//	Try to initialize CommandString from default device value
			def_prop = ds_class->get_default_device_property(dev_prop[i].name);
			if (def_prop.is_empty()==false)	def_prop  >>  commandString;
		}
		//	And try to extract CommandString value from database
		if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  commandString;

		//	Try to initialize BunchNumberString from class property
		cl_prop = ds_class->get_class_property(dev_prop[++i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  bunchNumberString;
		else {
			//	Try to initialize BunchNumberString from default device value
			def_prop = ds_class->get_default_device_property(dev_prop[i].name);
			if (def_prop.is_empty()==false)	def_prop  >>  bunchNumberString;
		}
		//	And try to extract BunchNumberString value from database
		if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  bunchNumberString;

	}

	/*----- PROTECTED REGION ID(DataVampire::get_device_property_after) ENABLED START -----*/
	/* clang-format on */
	//	Check device property data members init
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	DataVampire::get_device_property_after
}

//--------------------------------------------------------
/**
 *	Method      : DataVampire::always_executed_hook()
 *	Description : method always executed before any command is executed
 */
//--------------------------------------------------------
void DataVampire::always_executed_hook()
{
	DEBUG_STREAM << "DataVampire::always_executed_hook()  " << device_name << endl;
	/*----- PROTECTED REGION ID(DataVampire::always_executed_hook) ENABLED START -----*/
	/* clang-format on */
	//	code always executed before all requests
	/* clang-format off */
	
	
	/*----- PROTECTED REGION END -----*/	//	DataVampire::always_executed_hook
}

//--------------------------------------------------------
/**
 *	Method      : DataVampire::read_attr_hardware()
 *	Description : Hardware acquisition for attributes
 */
//--------------------------------------------------------
void DataVampire::read_attr_hardware(TANGO_UNUSED(vector<long> &attr_list))
{
	DEBUG_STREAM << "DataVampire::read_attr_hardware(vector<long> &attr_list) entering... " << endl;
	/*----- PROTECTED REGION ID(DataVampire::read_attr_hardware) ENABLED START -----*/
	/* clang-format on */
	//	Add your own code
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	DataVampire::read_attr_hardware
}
//--------------------------------------------------------
/**
 *	Method      : DataVampire::write_attr_hardware()
 *	Description : Hardware writing for attributes
 */
//--------------------------------------------------------
void DataVampire::write_attr_hardware(TANGO_UNUSED(vector<long> &attr_list))
{
	DEBUG_STREAM << "DataVampire::write_attr_hardware(vector<long> &attr_list) entering... " << endl;
	/*----- PROTECTED REGION ID(DataVampire::write_attr_hardware) ENABLED START -----*/
	/* clang-format on */
	//	Add your own code
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	DataVampire::write_attr_hardware
}

//--------------------------------------------------------
/**
 *	Read attribute MinThreshold related method
 *	Description: 
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void DataVampire::read_MinThreshold(Tango::Attribute &attr)
{
	DEBUG_STREAM << "DataVampire::read_MinThreshold(Tango::Attribute &attr) entering... " << endl;
	/*----- PROTECTED REGION ID(DataVampire::read_MinThreshold) ENABLED START -----*/
	/* clang-format on */
	//	Set the attribute value
	attr.set_value(attr_MinThreshold_read);
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	DataVampire::read_MinThreshold
}
//--------------------------------------------------------
/**
 *	Write attribute MinThreshold related method
 *	Description: 
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void DataVampire::write_MinThreshold(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "DataVampire::write_MinThreshold(Tango::WAttribute &attr) entering... " << endl;
	//	Retrieve write value
	Tango::DevDouble	w_val;
	attr.get_write_value(w_val);
	/*----- PROTECTED REGION ID(DataVampire::write_MinThreshold) ENABLED START -----*/
	/* clang-format on */
	//	Add your own code
	*attr_MinThreshold_read = w_val;
	
	
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	DataVampire::write_MinThreshold
}
//--------------------------------------------------------
/**
 *	Read attribute MaxThreshold related method
 *	Description: 
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void DataVampire::read_MaxThreshold(Tango::Attribute &attr)
{
	DEBUG_STREAM << "DataVampire::read_MaxThreshold(Tango::Attribute &attr) entering... " << endl;
	/*----- PROTECTED REGION ID(DataVampire::read_MaxThreshold) ENABLED START -----*/
	/* clang-format on */
	//	Set the attribute value
	attr.set_value(attr_MaxThreshold_read);
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	DataVampire::read_MaxThreshold
}
//--------------------------------------------------------
/**
 *	Write attribute MaxThreshold related method
 *	Description: 
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void DataVampire::write_MaxThreshold(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "DataVampire::write_MaxThreshold(Tango::WAttribute &attr) entering... " << endl;
	//	Retrieve write value
	Tango::DevDouble	w_val;
	attr.get_write_value(w_val);
	/*----- PROTECTED REGION ID(DataVampire::write_MaxThreshold) ENABLED START -----*/
	/* clang-format on */
	//	Add your own code
	*attr_MaxThreshold_read = w_val;
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	DataVampire::write_MaxThreshold
}
//--------------------------------------------------------
/**
 *	Read attribute CommandString related method
 *	Description: 
 *
 *	Data type:	Tango::DevString
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void DataVampire::read_CommandString(Tango::Attribute &attr)
{
	DEBUG_STREAM << "DataVampire::read_CommandString(Tango::Attribute &attr) entering... " << endl;
	/*----- PROTECTED REGION ID(DataVampire::read_CommandString) ENABLED START -----*/
	/* clang-format on */
	//	Set the attribute value
	attr.set_value(attr_CommandString_read);
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	DataVampire::read_CommandString
}
//--------------------------------------------------------
/**
 *	Write attribute CommandString related method
 *	Description: 
 *
 *	Data type:	Tango::DevString
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void DataVampire::write_CommandString(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "DataVampire::write_CommandString(Tango::WAttribute &attr) entering... " << endl;
	//	Retrieve write value
	Tango::DevString	w_val;
	attr.get_write_value(w_val);
	/*----- PROTECTED REGION ID(DataVampire::write_CommandString) ENABLED START -----*/
	/* clang-format on */
	//	Add your own code
	try {
		mutex->lock();
		std::vector<std::string> results;
		std::string new_configuration(w_val);
		string_explode(new_configuration, "/", &results);

		command_name = results[results.size()-1];
		size_t pos = new_configuration.find(command_name);
		if (pos != std::string::npos) { // if str2 found in str1
			new_configuration.erase(pos-1, command_name.length()+1); // erase str2 from str1
	    	}	
		c_device_name = new_configuration;

		if (device)
			delete device;
		device = new Tango::DeviceProxy(c_device_name);

		Tango::CommandInfo cmd_info = device->command_query(command_name);
		cmd_in_type = cmd_info.in_type;
		cmd_out_type = cmd_info.out_type;

		snprintf(*attr_CommandString_read, DATAVAMPIRE_MAX_STR_LEN, "%s", new_configuration.c_str());
		store_command_string(w_val);
		mutex->unlock();
	}
	catch (Tango::DevFailed &e) {
		mutex->unlock();
		INFO_STREAM << e.errors[0].desc << ", " << c_device_name << " " << command_name << endl;
		INFO_STREAM << e.errors[0].reason << ", " << c_device_name << " " << command_name << endl;
		Tango::Except::re_throw_exception(e,
			(const char *) "Failed to store command string",
			(const char *) "Error storing command string",
			__FUNCTION__, Tango::ERR);		
	}
	catch(...) {
		mutex->unlock();
		Tango::Except::throw_exception(
			(const char *) "Failed to store command string",
			(const char *) "Generic error",
			__FUNCTION__, Tango::ERR);		
	}		
	
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	DataVampire::write_CommandString
}
//--------------------------------------------------------
/**
 *	Read attribute PollingPeriod related method
 *	Description: -1: disabled
 *
 *	Data type:	Tango::DevLong
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void DataVampire::read_PollingPeriod(Tango::Attribute &attr)
{
	DEBUG_STREAM << "DataVampire::read_PollingPeriod(Tango::Attribute &attr) entering... " << endl;
	/*----- PROTECTED REGION ID(DataVampire::read_PollingPeriod) ENABLED START -----*/
	//	Set the attribute value
	attr.set_value(attr_PollingPeriod_read);
	
	/*----- PROTECTED REGION END -----*/	//	DataVampire::read_PollingPeriod
}
//--------------------------------------------------------
/**
 *	Write attribute PollingPeriod related method
 *	Description: -1: disabled
 *
 *	Data type:	Tango::DevLong
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void DataVampire::write_PollingPeriod(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "DataVampire::write_PollingPeriod(Tango::WAttribute &attr) entering... " << endl;
	//	Retrieve write value
	Tango::DevLong	w_val;
	attr.get_write_value(w_val);
	/*----- PROTECTED REGION ID(DataVampire::write_PollingPeriod) ENABLED START -----*/
	*attr_PollingPeriod_read = w_val;
	
	/*----- PROTECTED REGION END -----*/	//	DataVampire::write_PollingPeriod
}
//--------------------------------------------------------
/**
 *	Read attribute PollingSamples related method
 *	Description: -1: disabled
 *
 *	Data type:	Tango::DevLong
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void DataVampire::read_PollingSamples(Tango::Attribute &attr)
{
	DEBUG_STREAM << "DataVampire::read_PollingSamples(Tango::Attribute &attr) entering... " << endl;
	/*----- PROTECTED REGION ID(DataVampire::read_PollingSamples) ENABLED START -----*/
	/* clang-format on */
	//	Set the attribute value
	attr.set_value(attr_PollingSamples_read);
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	DataVampire::read_PollingSamples
}
//--------------------------------------------------------
/**
 *	Write attribute PollingSamples related method
 *	Description: -1: disabled
 *
 *	Data type:	Tango::DevLong
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void DataVampire::write_PollingSamples(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "DataVampire::write_PollingSamples(Tango::WAttribute &attr) entering... " << endl;
	//	Retrieve write value
	Tango::DevLong	w_val;
	attr.get_write_value(w_val);
	/*----- PROTECTED REGION ID(DataVampire::write_PollingSamples) ENABLED START -----*/
	/* clang-format on */
	//	Add your own code
	/* clang-format off */
	*attr_PollingSamples_read = w_val;
	/*----- PROTECTED REGION END -----*/	//	DataVampire::write_PollingSamples
}
//--------------------------------------------------------
/**
 *	Read attribute Sum related method
 *	Description: 
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void DataVampire::read_Sum(Tango::Attribute &attr)
{
	DEBUG_STREAM << "DataVampire::read_Sum(Tango::Attribute &attr) entering... " << endl;
	/*----- PROTECTED REGION ID(DataVampire::read_Sum) ENABLED START -----*/
	/* clang-format on */
	//	Set the attribute value
	attr.set_value(attr_Sum_read);
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	DataVampire::read_Sum
}
//--------------------------------------------------------
/**
 *	Read attribute Mean related method
 *	Description: 
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void DataVampire::read_Mean(Tango::Attribute &attr)
{
	DEBUG_STREAM << "DataVampire::read_Mean(Tango::Attribute &attr) entering... " << endl;
	/*----- PROTECTED REGION ID(DataVampire::read_Mean) ENABLED START -----*/
	/* clang-format on */
	//	Set the attribute value
	attr.set_value(attr_Mean_read);
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	DataVampire::read_Mean
}
//--------------------------------------------------------
/**
 *	Read attribute HorPos related method
 *	Description: 
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void DataVampire::read_HorPos(Tango::Attribute &attr)
{
	DEBUG_STREAM << "DataVampire::read_HorPos(Tango::Attribute &attr) entering... " << endl;
	/*----- PROTECTED REGION ID(DataVampire::read_HorPos) ENABLED START -----*/
	/* clang-format on */
	//	Set the attribute value
	attr.set_value(attr_HorPos_read);
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	DataVampire::read_HorPos
}
//--------------------------------------------------------
/**
 *	Read attribute VerPos related method
 *	Description: 
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void DataVampire::read_VerPos(Tango::Attribute &attr)
{
	DEBUG_STREAM << "DataVampire::read_VerPos(Tango::Attribute &attr) entering... " << endl;
	/*----- PROTECTED REGION ID(DataVampire::read_VerPos) ENABLED START -----*/
	/* clang-format on */
	//	Set the attribute value
	attr.set_value(attr_VerPos_read);
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	DataVampire::read_VerPos
}
//--------------------------------------------------------
/**
 *	Read attribute HorPosStd related method
 *	Description: 
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void DataVampire::read_HorPosStd(Tango::Attribute &attr)
{
	DEBUG_STREAM << "DataVampire::read_HorPosStd(Tango::Attribute &attr) entering... " << endl;
	/*----- PROTECTED REGION ID(DataVampire::read_HorPosStd) ENABLED START -----*/
	//	Set the attribute value
	attr.set_value(attr_HorPosStd_read);
	
	/*----- PROTECTED REGION END -----*/	//	DataVampire::read_HorPosStd
}
//--------------------------------------------------------
/**
 *	Read attribute ValidValues related method
 *	Description: 
 *
 *	Data type:	Tango::DevLong
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void DataVampire::read_ValidValues(Tango::Attribute &attr)
{
	DEBUG_STREAM << "DataVampire::read_ValidValues(Tango::Attribute &attr) entering... " << endl;
	/*----- PROTECTED REGION ID(DataVampire::read_ValidValues) ENABLED START -----*/
	/* clang-format on */
	//	Set the attribute value
	attr.set_value(attr_ValidValues_read);
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	DataVampire::read_ValidValues
}
//--------------------------------------------------------
/**
 *	Read attribute EnableAcquisition related method
 *	Description: 
 *
 *	Data type:	Tango::DevBoolean
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void DataVampire::read_EnableAcquisition(Tango::Attribute &attr)
{
	DEBUG_STREAM << "DataVampire::read_EnableAcquisition(Tango::Attribute &attr) entering... " << endl;
	/*----- PROTECTED REGION ID(DataVampire::read_EnableAcquisition) ENABLED START -----*/
	//	Set the attribute value
	attr.set_value(attr_EnableAcquisition_read);
	
	/*----- PROTECTED REGION END -----*/	//	DataVampire::read_EnableAcquisition
}
//--------------------------------------------------------
/**
 *	Write attribute EnableAcquisition related method
 *	Description: 
 *
 *	Data type:	Tango::DevBoolean
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void DataVampire::write_EnableAcquisition(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "DataVampire::write_EnableAcquisition(Tango::WAttribute &attr) entering... " << endl;
	//	Retrieve write value
	Tango::DevBoolean	w_val;
	attr.get_write_value(w_val);
	/*----- PROTECTED REGION ID(DataVampire::write_EnableAcquisition) ENABLED START -----*/
	*attr_EnableAcquisition_read = w_val;
	
	/*----- PROTECTED REGION END -----*/	//	DataVampire::write_EnableAcquisition
}
//--------------------------------------------------------
/**
 *	Read attribute CacheSize related method
 *	Description: 
 *
 *	Data type:	Tango::DevLong
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void DataVampire::read_CacheSize(Tango::Attribute &attr)
{
	DEBUG_STREAM << "DataVampire::read_CacheSize(Tango::Attribute &attr) entering... " << endl;
	/*----- PROTECTED REGION ID(DataVampire::read_CacheSize) ENABLED START -----*/
	/* clang-format on */
	//	Set the attribute value
	attr.set_value(attr_CacheSize_read);
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	DataVampire::read_CacheSize
}
//--------------------------------------------------------
/**
 *	Write attribute CacheSize related method
 *	Description: 
 *
 *	Data type:	Tango::DevLong
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void DataVampire::write_CacheSize(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "DataVampire::write_CacheSize(Tango::WAttribute &attr) entering... " << endl;
	//	Retrieve write value
	Tango::DevLong	w_val;
	attr.get_write_value(w_val);
	/*----- PROTECTED REGION ID(DataVampire::write_CacheSize) ENABLED START -----*/
	/* clang-format on */
	//	Add your own code
	*attr_CacheSize_read = w_val;

	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	DataVampire::write_CacheSize
}
//--------------------------------------------------------
/**
 *	Read attribute AutoBackgroundSamples related method
 *	Description: 0 to disable
 *
 *	Data type:	Tango::DevLong
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void DataVampire::read_AutoBackgroundSamples(Tango::Attribute &attr)
{
	DEBUG_STREAM << "DataVampire::read_AutoBackgroundSamples(Tango::Attribute &attr) entering... " << endl;
	/*----- PROTECTED REGION ID(DataVampire::read_AutoBackgroundSamples) ENABLED START -----*/
	/* clang-format on */
	//	Set the attribute value
	attr.set_value(attr_AutoBackgroundSamples_read);
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	DataVampire::read_AutoBackgroundSamples
}
//--------------------------------------------------------
/**
 *	Write attribute AutoBackgroundSamples related method
 *	Description: 0 to disable
 *
 *	Data type:	Tango::DevLong
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void DataVampire::write_AutoBackgroundSamples(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "DataVampire::write_AutoBackgroundSamples(Tango::WAttribute &attr) entering... " << endl;
	//	Retrieve write value
	Tango::DevLong	w_val;
	attr.get_write_value(w_val);
	/*----- PROTECTED REGION ID(DataVampire::write_AutoBackgroundSamples) ENABLED START -----*/
	/* clang-format on */
	//	Add your own code
	/* clang-format off */
	*attr_AutoBackgroundSamples_read = w_val;

	/*----- PROTECTED REGION END -----*/	//	DataVampire::write_AutoBackgroundSamples
}
//--------------------------------------------------------
/**
 *	Read attribute PostAcqRoi related method
 *	Description: 
 *
 *	Data type:	Tango::DevBoolean
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void DataVampire::read_PostAcqRoi(Tango::Attribute &attr)
{
	DEBUG_STREAM << "DataVampire::read_PostAcqRoi(Tango::Attribute &attr) entering... " << endl;
	/*----- PROTECTED REGION ID(DataVampire::read_PostAcqRoi) ENABLED START -----*/
	/* clang-format on */
	//	Set the attribute value
	attr.set_value(attr_PostAcqRoi_read);
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	DataVampire::read_PostAcqRoi
}
//--------------------------------------------------------
/**
 *	Write attribute PostAcqRoi related method
 *	Description: 
 *
 *	Data type:	Tango::DevBoolean
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void DataVampire::write_PostAcqRoi(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "DataVampire::write_PostAcqRoi(Tango::WAttribute &attr) entering... " << endl;
	//	Retrieve write value
	Tango::DevBoolean	w_val;
	attr.get_write_value(w_val);
	/*----- PROTECTED REGION ID(DataVampire::write_PostAcqRoi) ENABLED START -----*/
	/* clang-format on */
	//	Add your own code
	/* clang-format off */
	*attr_PostAcqRoi_read = w_val;

	/*----- PROTECTED REGION END -----*/	//	DataVampire::write_PostAcqRoi
}
//--------------------------------------------------------
/**
 *	Read attribute Width related method
 *	Description: 
 *
 *	Data type:	Tango::DevLong
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void DataVampire::read_Width(Tango::Attribute &attr)
{
	DEBUG_STREAM << "DataVampire::read_Width(Tango::Attribute &attr) entering... " << endl;
	/*----- PROTECTED REGION ID(DataVampire::read_Width) ENABLED START -----*/
	//	Set the attribute value
	attr.set_value(attr_Width_read);
	
	/*----- PROTECTED REGION END -----*/	//	DataVampire::read_Width
}
//--------------------------------------------------------
/**
 *	Write attribute Width related method
 *	Description: 
 *
 *	Data type:	Tango::DevLong
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void DataVampire::write_Width(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "DataVampire::write_Width(Tango::WAttribute &attr) entering... " << endl;
	//	Retrieve write value
	Tango::DevLong	w_val;
	attr.get_write_value(w_val);
	/*----- PROTECTED REGION ID(DataVampire::write_Width) ENABLED START -----*/
	*attr_Width_read = w_val;	
	
	/*----- PROTECTED REGION END -----*/	//	DataVampire::write_Width
}
//--------------------------------------------------------
/**
 *	Read attribute Height related method
 *	Description: 
 *
 *	Data type:	Tango::DevLong
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void DataVampire::read_Height(Tango::Attribute &attr)
{
	DEBUG_STREAM << "DataVampire::read_Height(Tango::Attribute &attr) entering... " << endl;
	/*----- PROTECTED REGION ID(DataVampire::read_Height) ENABLED START -----*/
	//	Set the attribute value
	attr.set_value(attr_Height_read);
	
	/*----- PROTECTED REGION END -----*/	//	DataVampire::read_Height
}
//--------------------------------------------------------
/**
 *	Write attribute Height related method
 *	Description: 
 *
 *	Data type:	Tango::DevLong
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void DataVampire::write_Height(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "DataVampire::write_Height(Tango::WAttribute &attr) entering... " << endl;
	//	Retrieve write value
	Tango::DevLong	w_val;
	attr.get_write_value(w_val);
	/*----- PROTECTED REGION ID(DataVampire::write_Height) ENABLED START -----*/
	*attr_Height_read = w_val;
	
	/*----- PROTECTED REGION END -----*/	//	DataVampire::write_Height
}
//--------------------------------------------------------
/**
 *	Read attribute VerPosStd related method
 *	Description: 
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void DataVampire::read_VerPosStd(Tango::Attribute &attr)
{
	DEBUG_STREAM << "DataVampire::read_VerPosStd(Tango::Attribute &attr) entering... " << endl;
	/*----- PROTECTED REGION ID(DataVampire::read_VerPosStd) ENABLED START -----*/
	//	Set the attribute value
	attr.set_value(attr_VerPosStd_read);
	
	/*----- PROTECTED REGION END -----*/	//	DataVampire::read_VerPosStd
}
//--------------------------------------------------------
/**
 *	Read attribute EnablePolling related method
 *	Description: 
 *
 *	Data type:	Tango::DevBoolean
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void DataVampire::read_EnablePolling(Tango::Attribute &attr)
{
	DEBUG_STREAM << "DataVampire::read_EnablePolling(Tango::Attribute &attr) entering... " << endl;
	/*----- PROTECTED REGION ID(DataVampire::read_EnablePolling) ENABLED START -----*/
	//	Set the attribute value
	attr.set_value(attr_EnablePolling_read);
	
	/*----- PROTECTED REGION END -----*/	//	DataVampire::read_EnablePolling
}
//--------------------------------------------------------
/**
 *	Write attribute EnablePolling related method
 *	Description: 
 *
 *	Data type:	Tango::DevBoolean
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void DataVampire::write_EnablePolling(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "DataVampire::write_EnablePolling(Tango::WAttribute &attr) entering... " << endl;
	//	Retrieve write value
	Tango::DevBoolean	w_val;
	attr.get_write_value(w_val);
	/*----- PROTECTED REGION ID(DataVampire::write_EnablePolling) ENABLED START -----*/
	*attr_EnablePolling_read = w_val;
	
	/*----- PROTECTED REGION END -----*/	//	DataVampire::write_EnablePolling
}
//--------------------------------------------------------
/**
 *	Read attribute BackgroundSamples related method
 *	Description: 0 to disable
 *
 *	Data type:	Tango::DevLong
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void DataVampire::read_BackgroundSamples(Tango::Attribute &attr)
{
	DEBUG_STREAM << "DataVampire::read_BackgroundSamples(Tango::Attribute &attr) entering... " << endl;
	/*----- PROTECTED REGION ID(DataVampire::read_BackgroundSamples) ENABLED START -----*/
	//	Set the attribute value
	attr.set_value(attr_BackgroundSamples_read);
	
	/*----- PROTECTED REGION END -----*/	//	DataVampire::read_BackgroundSamples
}
//--------------------------------------------------------
/**
 *	Write attribute BackgroundSamples related method
 *	Description: 0 to disable
 *
 *	Data type:	Tango::DevLong
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void DataVampire::write_BackgroundSamples(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "DataVampire::write_BackgroundSamples(Tango::WAttribute &attr) entering... " << endl;
	//	Retrieve write value
	Tango::DevLong	w_val;
	attr.get_write_value(w_val);
	/*----- PROTECTED REGION ID(DataVampire::write_BackgroundSamples) ENABLED START -----*/
	*attr_BackgroundSamples_read = w_val;	
	
	/*----- PROTECTED REGION END -----*/	//	DataVampire::write_BackgroundSamples
}
//--------------------------------------------------------
/**
 *	Read attribute RoiParam related method
 *	Description: 
 *
 *	Data type:	Tango::DevLong
 *	Attr type:	Spectrum max = 4
 */
//--------------------------------------------------------
void DataVampire::read_RoiParam(Tango::Attribute &attr)
{
	DEBUG_STREAM << "DataVampire::read_RoiParam(Tango::Attribute &attr) entering... " << endl;
	/*----- PROTECTED REGION ID(DataVampire::read_RoiParam) ENABLED START -----*/
	/* clang-format on */
	//	Set the attribute value
	attr.set_value(attr_RoiParam_read, roi_dim);
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	DataVampire::read_RoiParam
}
//--------------------------------------------------------
/**
 *	Write attribute RoiParam related method
 *	Description: 
 *
 *	Data type:	Tango::DevLong
 *	Attr type:	Spectrum max = 4
 */
//--------------------------------------------------------
void DataVampire::write_RoiParam(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "DataVampire::write_RoiParam(Tango::WAttribute &attr) entering... " << endl;
	//	Retrieve number of write values
	int	w_length = attr.get_write_value_length();

	//	Retrieve pointer on write values (Do not delete !)
	const Tango::DevLong	*w_val;
	attr.get_write_value(w_val);
	/*----- PROTECTED REGION ID(DataVampire::write_RoiParam) ENABLED START -----*/
	/* clang-format on */
	//	Add your own code
	/* clang-format off */

	if ((w_length == 2) && (w_val[0] > w_val[1])) {
		Tango::Except::throw_exception (
			(const char *)"Failed to set ROI param",
			(const char *)"Invalid setting",
			__FUNCTION__);		
	}
	if ((w_length == 4) && ((w_val[2] < w_val[0]) || (w_val[3] < w_val[1]))) {
		Tango::Except::throw_exception (
			(const char *)"Failed to set ROI param",
			(const char *)"Invalid setting",
			__FUNCTION__);		
	}
	
	store_roi_param(w_val, w_length);

	/*----- PROTECTED REGION END -----*/	//	DataVampire::write_RoiParam
}
//--------------------------------------------------------
/**
 *	Read attribute Spectrum related method
 *	Description: 
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Spectrum max = 1000000
 */
//--------------------------------------------------------
void DataVampire::read_Spectrum(Tango::Attribute &attr)
{
	DEBUG_STREAM << "DataVampire::read_Spectrum(Tango::Attribute &attr) entering... " << endl;
	/*----- PROTECTED REGION ID(DataVampire::read_Spectrum) ENABLED START -----*/
	/* clang-format on */
	//	Set the attribute value

	if (roi_dim == 4) {	
		Tango::Except::throw_exception(
			(const char *) "Failed to get spectrum",
			(const char *) "Dimension not supported",
			__FUNCTION__, Tango::ERR);	
	}
	
	attr.set_value(attr_Spectrum_read, num_cols_spectrum);
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	DataVampire::read_Spectrum
}
//--------------------------------------------------------
/**
 *	Read attribute HorBackground related method
 *	Description: 
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Spectrum max = 1000000
 */
//--------------------------------------------------------
void DataVampire::read_HorBackground(Tango::Attribute &attr)
{
	DEBUG_STREAM << "DataVampire::read_HorBackground(Tango::Attribute &attr) entering... " << endl;
	/*----- PROTECTED REGION ID(DataVampire::read_HorBackground) ENABLED START -----*/
	/* clang-format on */
	//	Set the attribute value
	int hor_size = 0;
	if (roi_dim == 2)
		hor_size = attr_RoiParam_read[1] - attr_RoiParam_read[0] + 1;
	else if (roi_dim == 4)
		hor_size = attr_RoiParam_read[2] - attr_RoiParam_read[0] + 1;

	attr.set_value(attr_HorBackground_read, hor_size);
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	DataVampire::read_HorBackground
}
//--------------------------------------------------------
/**
 *	Read attribute VerBackground related method
 *	Description: 
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Spectrum max = 1000000
 */
//--------------------------------------------------------
void DataVampire::read_VerBackground(Tango::Attribute &attr)
{
	DEBUG_STREAM << "DataVampire::read_VerBackground(Tango::Attribute &attr) entering... " << endl;
	/*----- PROTECTED REGION ID(DataVampire::read_VerBackground) ENABLED START -----*/
	/* clang-format on */
	//	Set the attribute value
	int ver_size = 0;
	if (roi_dim == 2)
		ver_size = 0;
	else if (roi_dim == 4)
		ver_size = attr_RoiParam_read[3] - attr_RoiParam_read[1] + 1;

	attr.set_value(attr_VerBackground_read, ver_size);
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	DataVampire::read_VerBackground
}
//--------------------------------------------------------
/**
 *	Read attribute Image related method
 *	Description: 
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Image max = 5000 x 5000
 */
//--------------------------------------------------------
void DataVampire::read_Image(Tango::Attribute &attr)
{
	DEBUG_STREAM << "DataVampire::read_Image(Tango::Attribute &attr) entering... " << endl;
	/*----- PROTECTED REGION ID(DataVampire::read_Image) ENABLED START -----*/
	/* clang-format on */
	//	Set the attribute value
	if (roi_dim == 2) {	
		Tango::Except::throw_exception(
			(const char *) "Failed to get spectrum",
			(const char *) "Dimension not supported",
			__FUNCTION__, Tango::ERR);	
	}	
	
	attr.set_value(attr_Image_read, num_cols_image, num_rows_image);
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	DataVampire::read_Image
}

//--------------------------------------------------------
/**
 *	Method      : DataVampire::add_dynamic_attributes()
 *	Description : Create the dynamic attributes if any
 *                for specified device.
 */
//--------------------------------------------------------
void DataVampire::add_dynamic_attributes()
{
	/*----- PROTECTED REGION ID(DataVampire::add_dynamic_attributes) ENABLED START -----*/
	/* clang-format on */
	//	Add your own code to create and add dynamic attributes if any
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	DataVampire::add_dynamic_attributes
}

//--------------------------------------------------------
/**
 *	Command GetSum related method
 *	Description: 
 *
 *	@param argin mode,samples; mode,bunch start,bunch end
 *	@returns values
 */
//--------------------------------------------------------
Tango::DevVarDoubleArray *DataVampire::get_sum(const Tango::DevVarLongArray *argin)
{
	Tango::DevVarDoubleArray *argout;
	DEBUG_STREAM << "DataVampire::GetSum()  - " << device_name << endl;
	/*----- PROTECTED REGION ID(DataVampire::get_sum) ENABLED START -----*/
	/* clang-format on */

	//	Add your own code
	try {
		mutex->lock();
		int32_t mode, num_samples, bn_start, bn_end;
	
		// check and extract input parameters
		get_param_in(argin, &mode, &num_samples, &bn_start, &bn_end);	
		// if data not preset get data and store it into circular buffers
		if ((bn_end > last_bn) || (mode == 0))
			store_circ_data(mode, num_samples, last_bn+1, bn_end);	
		// get start idx of the circular buffer
		int32_t start_idx = get_start_index(mode, num_samples, bn_start, bn_end);	
		argout = new Tango::DevVarDoubleArray();
		argout->length(num_samples);

		for (int32_t i = 0; i < num_samples; i++) {
			int32_t idx = (start_idx + i) % DATAVAMPIRE_MAX_SIZE;
			// get sum
			(*argout)[i] = acq_array[idx].sum;
		}
		mutex->unlock();	
	}
	catch (Tango::DevFailed &e) {
		mutex->unlock();
		Tango::Except::re_throw_exception(e,
			(const char *) "Failed to get data",
			(const char *) "Tango exception",
			__FUNCTION__, Tango::ERR);	
	}
	catch (...) {
		mutex->unlock();
		Tango::Except::throw_exception(
			(const char *) "Failed to get data",
			(const char *) "Generic exception",
			__FUNCTION__, Tango::ERR);	
	}
	mutex->unlock();		

	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	DataVampire::get_sum
	return argout;
}
//--------------------------------------------------------
/**
 *	Command GetMean related method
 *	Description: 
 *
 *	@param argin mode,samples; mode,bunch start,bunch end
 *	@returns values
 */
//--------------------------------------------------------
Tango::DevVarDoubleArray *DataVampire::get_mean(const Tango::DevVarLongArray *argin)
{
	Tango::DevVarDoubleArray *argout;
	DEBUG_STREAM << "DataVampire::GetMean()  - " << device_name << endl;
	/*----- PROTECTED REGION ID(DataVampire::get_mean) ENABLED START -----*/
	/* clang-format on */

	//	Add your own code
	try {
		mutex->lock();
		int32_t mode, num_samples, bn_start, bn_end;
		// check and extract input parameters
		get_param_in(argin, &mode, &num_samples, &bn_start, &bn_end);
		
		// if data not preset get data and store it into circular buffers	
		if ((bn_end > last_bn) || (mode == 0))
			store_circ_data(mode, num_samples, last_bn+1, bn_end);	
		
		// get start idx of the circular buffer
		int32_t start_idx = get_start_index(mode, num_samples, bn_start, bn_end);
		
		argout = new Tango::DevVarDoubleArray();
		argout->length(num_samples);
		
		for (int32_t i = 0; i < num_samples; i++) {
			int32_t idx = (start_idx + i) % DATAVAMPIRE_MAX_SIZE;
			// get mean
			(*argout)[i] = acq_array[idx].mean;
		}
		mutex->unlock();	
	}
	catch (Tango::DevFailed &e) {
		mutex->unlock();
		Tango::Except::re_throw_exception(e,
			(const char *) "Failed to get data",
			(const char *) "Tango exception",
			__FUNCTION__, Tango::ERR);	
	}
	catch (...) {
		mutex->unlock();
		Tango::Except::throw_exception(
			(const char *) "Failed to get data",
			(const char *) "Generic exception",
			__FUNCTION__, Tango::ERR);	
	}
	mutex->unlock();

	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	DataVampire::get_mean
	return argout;
}
//--------------------------------------------------------
/**
 *	Command GetHorPos related method
 *	Description: 
 *
 *	@param argin mode,samples; mode,bunch start,bunch end
 *	@returns values
 */
//--------------------------------------------------------
Tango::DevVarDoubleArray *DataVampire::get_hor_pos(const Tango::DevVarLongArray *argin)
{
	Tango::DevVarDoubleArray *argout;
	DEBUG_STREAM << "DataVampire::GetHorPos()  - " << device_name << endl;
	/*----- PROTECTED REGION ID(DataVampire::get_hor_pos) ENABLED START -----*/
	/* clang-format on */

	//	Add your own code
	try {
		mutex->lock();
		int32_t mode, num_samples, bn_start, bn_end;
		// check and extract input parameters
		get_param_in(argin, &mode, &num_samples, &bn_start, &bn_end);
		
		// if data not preset get data and store it into circular buffers	
		if ((bn_end > last_bn) || (mode == 0))
			store_circ_data(mode, num_samples, last_bn+1, bn_end);	
		
		// get start idx of the circular buffer
		int32_t start_idx = get_start_index(mode, num_samples, bn_start, bn_end);
		
		argout = new Tango::DevVarDoubleArray();
		argout->length(num_samples);
		
		for (int32_t i = 0; i < num_samples; i++) {
			int32_t idx = (start_idx + i) % DATAVAMPIRE_MAX_SIZE;
			// get horpos
			(*argout)[i] = acq_array[idx].horpos;
		}
		mutex->unlock();	
	}
	catch (Tango::DevFailed &e) {
		mutex->unlock();
		Tango::Except::re_throw_exception(e,
			(const char *) "Failed to get data",
			(const char *) "Tango exception",
			__FUNCTION__, Tango::ERR);	
	}
	catch (...) {
		mutex->unlock();
		Tango::Except::throw_exception(
			(const char *) "Failed to get data",
			(const char *) "Generic exception",
			__FUNCTION__, Tango::ERR);	
	}
	mutex->unlock();

	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	DataVampire::get_hor_pos
	return argout;
}
//--------------------------------------------------------
/**
 *	Command GetVerPos related method
 *	Description: 
 *
 *	@param argin mode,samples; mode,bunch start,bunch end
 *	@returns values
 */
//--------------------------------------------------------
Tango::DevVarDoubleArray *DataVampire::get_ver_pos(const Tango::DevVarLongArray *argin)
{
	Tango::DevVarDoubleArray *argout;
	DEBUG_STREAM << "DataVampire::GetVerPos()  - " << device_name << endl;
	/*----- PROTECTED REGION ID(DataVampire::get_ver_pos) ENABLED START -----*/
	/* clang-format on */

	//	Add your own code
	try {
		mutex->lock();
		int32_t mode, num_samples, bn_start, bn_end;
		// check and extract input parameters
		get_param_in(argin, &mode, &num_samples, &bn_start, &bn_end);
		
		// if data not preset get data and store it into circular buffers	
		if ((bn_end > last_bn) || (mode == 0))
			store_circ_data(mode, num_samples, last_bn+1, bn_end);	
		
		// get start idx of the circular buffer
		int32_t start_idx = get_start_index(mode, num_samples, bn_start, bn_end);
		
		argout = new Tango::DevVarDoubleArray();
		argout->length(num_samples);
		
		for (int32_t i = 0; i < num_samples; i++) {
			int32_t idx = (start_idx + i) % DATAVAMPIRE_MAX_SIZE;
			// get ve
			(*argout)[i] = acq_array[idx].verpos;
		}
		mutex->unlock();	
	}
	catch (Tango::DevFailed &e) {
		mutex->unlock();
		Tango::Except::re_throw_exception(e,
			(const char *) "Failed to get data",
			(const char *) "Tango exception",
			__FUNCTION__, Tango::ERR);	
	}
	catch (...) {
		mutex->unlock();
		Tango::Except::throw_exception(
			(const char *) "Failed to get data",
			(const char *) "Generic exception",
			__FUNCTION__, Tango::ERR);	
	}
	mutex->unlock();

	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	DataVampire::get_ver_pos
	return argout;
}
//--------------------------------------------------------
/**
 *	Command GetHorPosStd related method
 *	Description: 
 *
 *	@param argin mode,samples; mode,bunch start,bunch end
 *	@returns values
 */
//--------------------------------------------------------
Tango::DevVarDoubleArray *DataVampire::get_hor_pos_std(const Tango::DevVarLongArray *argin)
{
	Tango::DevVarDoubleArray *argout;
	DEBUG_STREAM << "DataVampire::GetHorPosStd()  - " << device_name << endl;
	/*----- PROTECTED REGION ID(DataVampire::get_hor_pos_std) ENABLED START -----*/
	/* clang-format on */

	//	Add your own code
	try {
		mutex->lock();
		int32_t mode, num_samples, bn_start, bn_end;
		// check and extract input parameters
		get_param_in(argin, &mode, &num_samples, &bn_start, &bn_end);
		
		// if data not preset get data and store it into circular buffers	
		if ((bn_end > last_bn) || (mode == 0))
			store_circ_data(mode, num_samples, last_bn+1, bn_end);	
		
		// get start idx of the circular buffer
		int32_t start_idx = get_start_index(mode, num_samples, bn_start, bn_end);
		
		argout = new Tango::DevVarDoubleArray();
		argout->length(num_samples);
		
		for (int32_t i = 0; i < num_samples; i++) {
			int32_t idx = (start_idx + i) % DATAVAMPIRE_MAX_SIZE;
			// get sum
			(*argout)[i] = acq_array[idx].horposstd;
		}
		mutex->unlock();	
	}
	catch (Tango::DevFailed &e) {
		mutex->unlock();
		Tango::Except::re_throw_exception(e,
			(const char *) "Failed to get data",
			(const char *) "Tango exception",
			__FUNCTION__, Tango::ERR);	
	}
	catch (...) {
		mutex->unlock();
		Tango::Except::throw_exception(
			(const char *) "Failed to get data",
			(const char *) "Generic exception",
			__FUNCTION__, Tango::ERR);	
	}
	mutex->unlock();

	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	DataVampire::get_hor_pos_std
	return argout;
}
//--------------------------------------------------------
/**
 *	Command GetVerPosStd related method
 *	Description: 
 *
 *	@param argin mode,samples; mode,bunch start,bunch end
 *	@returns values
 */
//--------------------------------------------------------
Tango::DevVarDoubleArray *DataVampire::get_ver_pos_std(const Tango::DevVarLongArray *argin)
{
	Tango::DevVarDoubleArray *argout;
	DEBUG_STREAM << "DataVampire::GetVerPosStd()  - " << device_name << endl;
	/*----- PROTECTED REGION ID(DataVampire::get_ver_pos_std) ENABLED START -----*/
	/* clang-format on */

	//	Add your own code
	try {
		mutex->lock();
		int32_t mode, num_samples, bn_start, bn_end;
		// check and extract input parameters
		get_param_in(argin, &mode, &num_samples, &bn_start, &bn_end);
		
		// if data not preset get data and store it into circular buffers	
		if ((bn_end > last_bn) || (mode == 0))
			store_circ_data(mode, num_samples, last_bn+1, bn_end);	
		
		// get start idx of the circular buffer
		int32_t start_idx = get_start_index(mode, num_samples, bn_start, bn_end);
		
		argout = new Tango::DevVarDoubleArray();
		argout->length(num_samples);
		
		for (int32_t i = 0; i < num_samples; i++) {
			int32_t idx = (start_idx + i) % DATAVAMPIRE_MAX_SIZE;
			// get sum
			(*argout)[i] = acq_array[idx].verposstd;
		}
		mutex->unlock();	
	}
	catch (Tango::DevFailed &e) {
		mutex->unlock();
		Tango::Except::re_throw_exception(e,
			(const char *) "Failed to get data",
			(const char *) "Tango exception",
			__FUNCTION__, Tango::ERR);	
	}
	catch (...) {
		mutex->unlock();
		Tango::Except::throw_exception(
			(const char *) "Failed to get data",
			(const char *) "Generic exception",
			__FUNCTION__, Tango::ERR);	
	}
	mutex->unlock();

	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	DataVampire::get_ver_pos_std
	return argout;
}
//--------------------------------------------------------
/**
 *	Command GetValidValues related method
 *	Description: 
 *
 *	@param argin mode,samples; mode,bunch start,bunch end
 *	@returns values
 */
//--------------------------------------------------------
Tango::DevVarDoubleArray *DataVampire::get_valid_values(const Tango::DevVarLongArray *argin)
{
	Tango::DevVarDoubleArray *argout;
	DEBUG_STREAM << "DataVampire::GetValidValues()  - " << device_name << endl;
	/*----- PROTECTED REGION ID(DataVampire::get_valid_values) ENABLED START -----*/
	/* clang-format on */

	//	Add your own code
	try {
		mutex->lock();
		int32_t mode, num_samples, bn_start, bn_end;
		// check and extract input parameters
		get_param_in(argin, &mode, &num_samples, &bn_start, &bn_end);
		
		// if data not preset get data and store it into circular buffers	
		if ((bn_end > last_bn) || (mode == 0))
			store_circ_data(mode, num_samples, last_bn+1, bn_end);	
		
		// get start idx of the circular buffer
		int32_t start_idx = get_start_index(mode, num_samples, bn_start, bn_end);
		
		argout = new Tango::DevVarDoubleArray();
		argout->length(num_samples);
		
		for (int32_t i = 0; i < num_samples; i++) {
			int32_t idx = (start_idx + i) % DATAVAMPIRE_MAX_SIZE;
			// get sum
			(*argout)[i] = acq_array[idx].validvalues;
		}
		mutex->unlock();	
	}
	catch (Tango::DevFailed &e) {
		mutex->unlock();
		Tango::Except::re_throw_exception(e,
			(const char *) "Failed to get data",
			(const char *) "Tango exception",
			__FUNCTION__, Tango::ERR);	
	}
	catch (...) {
		mutex->unlock();
		Tango::Except::throw_exception(
			(const char *) "Failed to get data",
			(const char *) "Generic exception",
			__FUNCTION__, Tango::ERR);	
	}
	mutex->unlock();
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	DataVampire::get_valid_values
	return argout;
}
//--------------------------------------------------------
/**
 *	Command SetBackground related method
 *	Description: 
 *
 */
//--------------------------------------------------------
void DataVampire::set_background()
{
	DEBUG_STREAM << "DataVampire::SetBackground()  - " << device_name << endl;
	/*----- PROTECTED REGION ID(DataVampire::set_background) ENABLED START -----*/
	/* clang-format on */

	//	Add your own code
	for (int i = 0; i < array_idx; i++) {
		if ((acq_array[i].bn == 0) || (acq_array[i].sum == 0)) {
			printf("%d bn=%d last_bn=%d sum=%f\n",i,acq_array[i].bn,last_bn,acq_array[i].sum);
		}	
	}
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	DataVampire::set_background
}
//--------------------------------------------------------
/**
 *	Method      : DataVampire::add_dynamic_commands()
 *	Description : Create the dynamic commands if any
 *                for specified device.
 */
//--------------------------------------------------------
void DataVampire::add_dynamic_commands()
{
	/*----- PROTECTED REGION ID(DataVampire::add_dynamic_commands) ENABLED START -----*/
	/* clang-format on */
	//	Add your own code to create and add dynamic commands if any
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	DataVampire::add_dynamic_commands
}

/*----- PROTECTED REGION ID(DataVampire::namespace_ending) ENABLED START -----*/
/* clang-format on */
//	Additional Methods
/* clang-format off */





void DataVampire::cmd_extract_values(Tango::DeviceData *cmd_value, std::vector<double> &val, std::vector<std::string> &val_string)
{

	Tango::DevState stval;
	std::vector<Tango::DevState> v_st;
	std::vector<Tango::DevULong> v_ulo;
	std::vector<Tango::DevUChar> v_uch;
	std::vector<Tango::DevShort> v_sh;
	std::vector<Tango::DevUShort> v_ush;
	std::vector<Tango::DevLong> v_lo;
	std::vector<Tango::DevDouble> v_do;
	std::vector<Tango::DevFloat> v_fl;
	std::vector<Tango::DevBoolean> v_bo;
	std::vector<Tango::DevLong64> v_lo64;
	std::vector<Tango::DevULong64> v_ulo64;
	//vector<string> v_string;

	long cmd_type = cmd_value->get_type();

	val.clear();

	if ((cmd_type == Tango::DEV_UCHAR) || (cmd_type == Tango::DEVVAR_CHARARRAY)) 
	{
		*cmd_value >> v_uch;
		for(std::vector<Tango::DevUChar>::iterator it = v_uch.begin(); it != v_uch.end(); it++)
			val.push_back((double)(*it));		//convert all to double
	}
	else if ((cmd_type == Tango::DEV_SHORT) || (cmd_type == Tango::DEVVAR_SHORTARRAY)) 
	{
		*cmd_value >> v_sh;
		for(std::vector<Tango::DevShort>::iterator  it = v_sh.begin(); it != v_sh.end(); it++)
			val.push_back((double)(*it));		//convert all to double
	}
	else if ((cmd_type == Tango::DEV_USHORT) || (cmd_type == Tango::DEVVAR_USHORTARRAY)) 
	{
		*cmd_value >> v_ush;
		for(std::vector<Tango::DevUShort>::iterator  it = v_ush.begin(); it != v_ush.end(); it++)
			val.push_back((double)(*it));		//convert all to double
	}
	else if ((cmd_type == Tango::DEV_LONG) || (cmd_type == Tango::DEVVAR_LONGARRAY)) 
	{
		*cmd_value >> v_lo;
		for(std::vector<Tango::DevLong>::iterator  it = v_lo.begin(); it != v_lo.end(); it++)
			val.push_back((double)(*it));		//convert all to double
	}
	else if ((cmd_type == Tango::DEV_LONG64) || (cmd_type == Tango::DEVVAR_LONG64ARRAY)) 
	{
		*cmd_value >> v_lo64;
		for(std::vector<Tango::DevLong64>::iterator  it = v_lo64.begin(); it != v_lo64.end(); it++)
			val.push_back((double)(*it));		//convert all to double
	}
	else if (cmd_type == Tango::DEV_STATE)
	{
		//*cmd_value >> v_st;
		//TODO: why extract_read not working with State
		std::cout << "GG: Do not compile cmd_value->get_type() == Tango::DEV_STATE ???" << std::endl;
		if(v_st.size() == 0)
		{
			*cmd_value >> stval;
			v_st.push_back(stval);
		}
		for(std::vector<Tango::DevState>::iterator it = v_st.begin(); it != v_st.end(); it++)
		{
			val.push_back((double)(*it));		//convert all to double
		}
	}
	else if ((cmd_type == Tango::DEV_ULONG) || (cmd_type == Tango::DEVVAR_ULONGARRAY))
	{
		*cmd_value >> v_ulo;
		for(std::vector<Tango::DevULong>::iterator  it = v_ulo.begin(); it != v_ulo.end(); it++)
			val.push_back((double)(*it));		//convert all to double
	}
	else if ((cmd_type == Tango::DEV_ULONG64) || (cmd_type == Tango::DEVVAR_ULONG64ARRAY)) 
	{
		*cmd_value >> v_ulo64;
		for(std::vector<Tango::DevULong64>::iterator  it = v_ulo64.begin(); it != v_ulo64.end(); it++)
			val.push_back((double)(*it));		//convert all to double
	}
	else if ((cmd_type == Tango::DEV_DOUBLE) || (cmd_type == Tango::DEVVAR_DOUBLEARRAY)) 
	{
		*cmd_value >> v_do;

		for(std::vector<Tango::DevDouble>::iterator  it = v_do.begin(); it != v_do.end(); it++)
			val.push_back((double)(*it));		//convert all to double
	}
	else if ((cmd_type == Tango::DEV_FLOAT) || (cmd_type == Tango::DEVVAR_FLOATARRAY)) 
	{
		*cmd_value >> v_fl;
		for(std::vector<Tango::DevFloat>::iterator  it = v_fl.begin(); it != v_fl.end(); it++)
			val.push_back((double)(*it));		//convert all to double
	}
	else if ((cmd_type == Tango::DEV_BOOLEAN) || (cmd_type == Tango::DEVVAR_BOOLEANARRAY)) 
	{
		//*cmd_value >> v_bo; // GG do not compile ????
		std::cout << "GG: Do not compile cmd_value->get_type() == Tango::DEV_BOOLEAN ???" << std::endl;
		for(std::vector<Tango::DevBoolean>::iterator  it = v_bo.begin(); it != v_bo.end(); it++)
			val.push_back((double)(*it));		//convert all to double
	}
	else if ((cmd_type == Tango::DEV_STRING) || (cmd_type == Tango::DEVVAR_STRINGARRAY)) 
	{
		/*attr_value->extract_read(v_string);
		val_string = *(v_string.begin());*/
		*cmd_value >> val_string;
	}
	else
	{
		std::ostringstream o;
		o << "unknown type" << std::ends;
		throw o.str();
	}

}


Tango::DeviceData DataVampire::cmd_insert_values(long cmd_type, std::vector<double> val, std::vector<std::string> val_string)
{
	Tango::DevState st;
	std::vector<Tango::DevState> v_st;
	std::vector<Tango::DevULong> v_ulo;
	Tango::DevULong ulo;
	std::vector<Tango::DevULong64> v_ulo64;
	Tango::DevULong64 ulo64;
	std::vector<Tango::DevUChar> v_uch;
	Tango::DevUChar uch;
	std::vector<Tango::DevShort> v_sh;
	Tango::DevShort sh;
	std::vector<Tango::DevUShort> v_ush;
	Tango::DevUShort ush;
	std::vector<Tango::DevLong> v_lo;
	Tango::DevLong lo;
	std::vector<Tango::DevLong64> v_lo64;
	Tango::DevLong64 lo64;
	std::vector<Tango::DevDouble> v_do;
	Tango::DevDouble _do;
	std::vector<Tango::DevFloat> v_fl;
	Tango::DevFloat fl;
	std::vector<Tango::DevBoolean> v_bo;
	Tango::DevBoolean bo;
	std::vector<std::string> v_string;
	std::string _string;
	Tango::DeviceData din_value;
	
	if ((cmd_type == Tango::DEV_UCHAR) || (cmd_type == Tango::DEVVAR_CHARARRAY)) 
	{
		if(val.size() > 1)
		{
			for(std::vector<double>::iterator it = val.begin(); it != val.end(); it++)
				v_uch.push_back((Tango::DevUChar)(*it));		//convert all from double
			//din_value << v_uch;
			std::cout << "GG: Do not compile cmd_type == Tango::DEV_UCHAR ???" << std::endl;
		}
		else
		{
			uch = (Tango::DevUChar)(*val.begin());
			// din_value << uch;
			std::cout << "GG: Do not compile cmd_type == Tango::DEV_UCHAR ???" << std::endl;
		}
	}
	else if ((cmd_type == Tango::DEV_SHORT) || (cmd_type == Tango::DEVVAR_SHORTARRAY)) 
	{
		if(val.size() > 1)
		{
			for(std::vector<double>::iterator it = val.begin(); it != val.end(); it++)
				v_sh.push_back((Tango::DevShort)(*it));		//convert all from double
			din_value << v_sh;
		}
		else
		{
			sh = (Tango::DevShort)(*val.begin());
			din_value << sh;
		}
	}
	else if ((cmd_type == Tango::DEV_USHORT) || (cmd_type == Tango::DEVVAR_USHORTARRAY)) 
	{
		if(val.size() > 1)
		{
			for(std::vector<double>::iterator it = val.begin(); it != val.end(); it++)
				v_ush.push_back((Tango::DevUShort)(*it));		//convert all from double
			din_value << v_ush;
		}
		else
		{
			ush = (Tango::DevUShort)(*val.begin());
			din_value << ush;
		}
	}
	else if ((cmd_type == Tango::DEV_LONG) || (cmd_type == Tango::DEVVAR_LONGARRAY)) 
	{
		if(val.size() > 1)
		{
			for(std::vector<double>::iterator it = val.begin(); it != val.end(); it++)
				v_lo.push_back((Tango::DevLong)(*it));		//convert all from double
			din_value << v_lo;
		}
		else
		{
			lo = (Tango::DevLong)(*val.begin());
			din_value << lo;
		}
	}
	else if ((cmd_type == Tango::DEV_LONG64) || (cmd_type == Tango::DEVVAR_LONG64ARRAY)) 
	{
		if(val.size() > 1)
		{
			for(std::vector<double>::iterator it = val.begin(); it != val.end(); it++)
				v_lo64.push_back((Tango::DevLong64)(*it));		//convert all from double
			din_value << v_lo64;
		}
		else
		{
			lo64 = (Tango::DevLong64)(*val.begin());
			din_value << lo64;
		}
	}
	else if (cmd_type == Tango::DEV_STATE)
	{
		if(val.size() > 1)
		{
			for(std::vector<double>::iterator it = val.begin(); it != val.end(); it++)
				v_st.push_back((Tango::DevState)(*it));		//convert all from double
			//din_value << v_st;
			std::cout << "GG: Do not compile cmd_type == Tango::DEV_STATE ???" << std::endl;
		}
		else
		{
			st = (Tango::DevState)(*(val.begin()));
			din_value << st;
		}
	}
	else if ((cmd_type == Tango::DEV_ULONG) || (cmd_type == Tango::DEVVAR_ULONGARRAY)) 
	{
		if(val.size() > 1)
		{
			for(std::vector<double>::iterator it = val.begin(); it != val.end(); it++)
				v_ulo.push_back((Tango::DevULong)(*it));		//convert all from double
			din_value << v_ulo;
		}
		else
		{
			ulo = (Tango::DevULong)(*(val.begin()));
			din_value << ulo;
		}
	}
	else if ((cmd_type == Tango::DEV_ULONG64) || (cmd_type == Tango::DEVVAR_ULONG64ARRAY)) 
	{
		if(val.size() > 1)
		{
			for(std::vector<double>::iterator it = val.begin(); it != val.end(); it++)
				v_ulo64.push_back((Tango::DevULong64)(*it));		//convert all from double
			din_value << v_ulo64;
		}
		else
		{
			ulo64 = (Tango::DevULong64)(*(val.begin()));
			din_value << ulo64;
		}
	}
	else if ((cmd_type == Tango::DEV_DOUBLE) || (cmd_type == Tango::DEVVAR_DOUBLEARRAY)) 
	{
		if(val.size() > 1)
		{
			for(std::vector<double>::iterator it = val.begin(); it != val.end(); it++)
			{
				v_do.push_back((Tango::DevDouble)(*it));		//convert all from double
			}
			din_value << v_do;
		}
		else
		{
			_do = (Tango::DevDouble)(*(val.begin()));
			din_value << _do;
		}
	}
	else if ((cmd_type == Tango::DEV_FLOAT) || (cmd_type == Tango::DEVVAR_FLOATARRAY)) 
	{
		if(val.size() > 1)
		{
			for(std::vector<double>::iterator it = val.begin(); it != val.end(); it++)
				v_fl.push_back((Tango::DevFloat)(*it));		//convert all from double
			din_value << v_fl;
		}
		else
		{
			fl = (Tango::DevFloat)(*(val.begin()));
			din_value << fl;
		}
	}
	else if ((cmd_type == Tango::DEV_BOOLEAN) || (cmd_type == Tango::DEVVAR_BOOLEANARRAY)) 
	{
		if(val.size() > 1)
		{
			for(std::vector<double>::iterator it = val.begin(); it != val.end(); it++)
				v_bo.push_back((Tango::DevBoolean)(*it));		//convert all from double
			// din_value << v_bo;
				std::cout << "GG: Do not compile cmd_type == Tango::DEV_BOOLEAN ???" << std::endl;
		}
		else
		{
			bo = (Tango::DevBoolean)(*(val.begin()));
			din_value << bo;
		}
	}
	else if ((cmd_type == Tango::DEV_STRING) || (cmd_type == Tango::DEVVAR_STRINGARRAY)) 
	{
		if(val.size() > 1)
		{
			for(std::vector<std::string>::iterator it = val_string.begin(); it != val_string.end(); it++)
				v_string.push_back(*it);		//convert all from string
			din_value << v_string;
		}
		else
		{
			_string = *(val_string.begin());
			din_value << _string;
		}
	}
	else
	{
		std::ostringstream o;
		o << "unknown type" << std::ends;
		throw o.str();
	}

	return din_value;
}


//+------------------------------------------------------------------
/**
 *	method:	DataVampire::string_explode
 * Explode a string 
*/
//+------------------------------------------------------------------
void DataVampire::string_explode(std::string str, std::string separator, std::vector<std::string>* results)
{
	size_t found;
	
	found = str.find_first_of(separator);
	while(found != std::string::npos) {
		if(found > 0) {
			results->push_back(str.substr(0,found));
		}
		str = str.substr(found+1);
		found = str.find_first_of(separator);
	}
	if(str.length() > 0) {
		results->push_back(str);
	}

}



//+------------------------------------------------------------------
/**
 * method:	DataVampire::store_roi_param
 * Store command string 
*/
//+------------------------------------------------------------------
void DataVampire::store_roi_param(const Tango::DevLong *w_val, int size)
{

	Tango::Database *db = new Tango::Database();
	char conf_str[20];

	/* manual ROI x1,y1,x2,y2 */
	try {
		mutex->lock();
		Tango::DbDatum config("RoiParam");
		Tango::DbData db_data_put;
		std::vector<std::string> val;
		for (int i = 0; i < size; i++) {
			memset(conf_str,0,20);
			sprintf(conf_str,"%d", w_val[i]);
			val.push_back(conf_str);
		}
		roi_dim = size;
		for (int i = size; i < 4; i++) {
			memset(conf_str,0,20);
			sprintf(conf_str,"-1");
			val.push_back(conf_str);
		}

		config << val;
		db_data_put.push_back(config);
		db->put_device_property(get_name(), db_data_put);
		
	}
	catch (Tango::DevFailed &e) {
		mutex->unlock();
		delete db;
		Tango::Except::throw_exception (
			(const char *)"Failed to update configuration",
			(const char *)"Can't write into database",
			__FUNCTION__);				
	}
	catch (...) {
		mutex->unlock();
		delete db;
		Tango::Except::throw_exception (
			(const char *)"Failed to update configuration",
			(const char *)"Unknown error",
			__FUNCTION__);		
	}
	delete db;

	for (int i = 0; i < size; i++) {
		attr_RoiParam_read[i] =  w_val[i];
	}
	for (int i = size; i < 4; i++) {
		attr_RoiParam_read[i] = -1;
	}

	roi_dim = size;
	
	mutex->unlock();

}


//+------------------------------------------------------------------
/**
 * method: DataVampire::store_command_string
 * Store command string 
*/
//+------------------------------------------------------------------
void DataVampire::store_command_string(Tango::DevString w_val)
{
	std::string new_configuration(w_val);
	std::string dev_property("CommandString");

	strncpy(*attr_CommandString_read, new_configuration.c_str(), 256);
	
	Tango::Database *db = new Tango::Database(); 
	
	try {
		std::string dev_property("CommandString");
		Tango::DbDatum config(dev_property);
		Tango::DbData db_data_del, db_data_put;
		db_data_del.push_back(config);
		/* start data */
		config << new_configuration;
		/* end data */
		db_data_put.push_back(config);
		db->put_device_property(get_name(), db_data_put);
	}
	catch (CORBA::Exception &e) {
		delete db;	
		ERROR_STREAM << "write_CommandString(): failed to save parameters" << std::endl;		
		Tango::Except::throw_exception (
			(const char *)"Failed to update configuration",
			(const char *)"Can't write into database",
			__FUNCTION__);
	}	
	delete db;

}


void DataVampire::store_array_2D(Tango::DeviceData *cmd_value, int32_t bn_start, int32_t bn_end)
{
	std::vector<double> val;
	std::vector<std::string> val_string;
	int32_t num_samples = bn_end - bn_start + 1;
	int32_t roi[4];
	int32_t auto_bck_samples = *attr_AutoBackgroundSamples_read;
	
	if (roi_dim != 4) {
		Tango::Except::throw_exception (
			(const char *)"Failed to store data",
			(const char *)"Invalid ROI dimension",
			__FUNCTION__);		
	}
	
	// minimize risks of anomalies in num_cols/rows calculation 
	memcpy(roi, attr_RoiParam_read, 4 * sizeof(int32_t));

	if (*attr_EnableAcquisition_read == false) {
		Tango::Except::throw_exception (
			(const char *)"Failed to store data",
			(const char *)"Acquisition is disabled",
			__FUNCTION__);	
	}

	cmd_extract_values(cmd_value,  val, val_string);

	int num_rows, num_cols;
	if (roi_dim == 2) {
		Tango::Except::throw_exception (
			(const char *)"Failed to store data",
			(const char *)"Invalid ROI setting",
			__FUNCTION__);	
	}
	else {
		num_rows = roi[3]-roi[1]+1,
		num_cols = roi[2]-roi[0]+1;
	}

	int32_t wave_size = val.size() / num_samples;

	if (((num_cols * num_rows * num_samples) != val.size()) && (*attr_PostAcqRoi_read == false)) {
		Tango::Except::throw_exception (
			(const char *)"Failed to store data",
			(const char *)"Data size does not match roi setting",
			__FUNCTION__);
	}

	if (((num_cols * num_rows * num_samples) > val.size()) && (*attr_PostAcqRoi_read == true)) {
		Tango::Except::throw_exception (
			(const char *)"Failed to store data",
			(const char *)"Data size too large",
			__FUNCTION__);
	}

	if ((*attr_PostAcqRoi_read == true) && (roi_dim == 4)) { 
		if (val.size() != (*attr_Width_read * *attr_Height_read * num_samples)) {
			Tango::Except::throw_exception (
				(const char *)"Failed to store data",
				(const char *)"Width/Height mismatch with image size",
				__FUNCTION__);		
		}
	
	}

	vector <bool> data_valid;
	
	if (*attr_PostAcqRoi_read == false) {
		for (int i = 0; i < val.size(); i++) {
			if ((val[i] >= *attr_MinThreshold_read) && (val[i] <= *attr_MaxThreshold_read)) 
				data_valid.push_back(true);
			else
				data_valid.push_back(false);
		}
	}
	// PostROI enabled
	else {
		int cnt = 0;
		for (int i = 0; i < num_samples; i++) {
			int offset = wave_size * i;
			// ImageWidth must be defined because command_inout does not report the Image width/length values
			if ((*attr_Width_read > 0) && (roi_dim == 4)) {
				for (int j = roi[1]; j <= roi[3]; j++) {
					for (int k = roi[0]; k <= roi[2]; k++) {
						val[cnt] = val[j * *attr_Width_read + k + offset];
						if ((val[cnt] >= *attr_MinThreshold_read) && 
							(val[cnt] <= *attr_MaxThreshold_read)) 
							data_valid.push_back(true);
						else
							data_valid.push_back(false);
						cnt++;
					}
				}
			}
		}
		wave_size = (double)cnt / (double)num_samples;
	}

	double *hor_profile_bck, *ver_profile_bck;
	uint32_t *hor_profile_bck_cnt, *ver_profile_bck_cnt;

	// for Image display
	num_rows_image = num_rows;
	num_cols_image = num_cols;

	hor_profile_bck = (double *) calloc(num_cols, sizeof(double));
	ver_profile_bck = (double *) calloc(num_rows, sizeof(double));
	hor_profile_bck_cnt = (uint32_t *) calloc(num_cols, sizeof(uint32_t));
	ver_profile_bck_cnt = (uint32_t *) calloc(num_rows, sizeof(uint32_t));

	for (int i = 0; i < num_samples; i++) {

		memset(hor_profile_bck, 0, sizeof(float) * num_cols);
		memset(ver_profile_bck, 0, sizeof(float) * num_rows);
		memset(hor_profile_bck_cnt, 0, sizeof(uint32_t) * num_cols);
		memset(ver_profile_bck_cnt, 0, sizeof(uint32_t) * num_rows);

		if (auto_bck_samples) {
			// calculate horizontal background profile from top and bottom auto_bck_samples rows
			for (int j = 0; j <  auto_bck_samples; j++) {			
				for (int k = 0; k < num_cols; k++) {
					int idx = j*num_cols+k+wave_size*i;
					if (data_valid[idx]) {
						hor_profile_bck[k] += val[idx];	
						hor_profile_bck_cnt[k]++;	
					}
					idx = (num_rows-j-1)*num_cols+k+wave_size*i;
					if (data_valid[idx]) {
						hor_profile_bck[k] += val[idx];	
						hor_profile_bck_cnt[k]++;	
					}
				}
			}

			// calculate vertical background profile from left and right auto_bck_samples columns
			for (int j = 0; j <  auto_bck_samples; j++) {
				// rows				
				for (int k = 0; k < num_rows; k++) {			
					int idx = k*num_cols+j+wave_size*i;
					if (data_valid[idx]) {
						ver_profile_bck[k] += val[idx];
						ver_profile_bck_cnt[k]++;	
					}
					int idx2 = idx + (num_cols - 1) - j;
					if (data_valid[idx2]) {
						ver_profile_bck[k] += val[idx2];	
						ver_profile_bck_cnt[k]++;	
					}
				}
			}
	

			for (int j = 0; j < num_cols; j++) {
				if (hor_profile_bck_cnt[j] > 0) {
					hor_profile_bck[j] = hor_profile_bck[j] / (double)hor_profile_bck_cnt[j];
				}
				else
					hor_profile_bck[j] = 0;
			}
			for (int j = 0; j < num_rows; j++) {
				if (ver_profile_bck_cnt[j] > 0) {
					ver_profile_bck[j] = ver_profile_bck[j] / (double)ver_profile_bck_cnt[j];
				}
				else
					ver_profile_bck[j] = 0;
			}

			if (i == 0) {
				for (int j = 0; j < num_cols; j++) 
					attr_HorBackground_read[j] = hor_profile_bck[j];
				for (int j = 0; j < num_rows; j++)
					attr_VerBackground_read[j] = ver_profile_bck[j];
			}

			// remove horizontal backgroud
			for (int j = 0; j < num_rows; j++) {
				for (int k = 0; k < num_cols; k++) {
					int idx = j * num_cols + k + wave_size*i;
					val[idx] -= hor_profile_bck[k];
				}
			}

			// remove vertical backgroud
			for (int j = 0; j < num_cols; j++) {
				for (int k = 0; k < num_rows; k++) {
					int idx = k * num_cols + j + wave_size*i;
					val[idx] -= ver_profile_bck[k];
				}
			}


			for (int j = 0; j < num_rows; j++) {
				for (int k = 0; k < num_cols; k++) {
					int idx = j * num_cols + k + wave_size*i;
					if (val[idx] < 0)
						val[idx] = 0;						
				}
			}
			
		}  // auto_bck_samples


		// calculate position, intensity and rms of the image
		double hor_pos = 0, ver_pos = 0, mean = 0;
		double std_hor_pos = 0, std_ver_pos = 0;
		double total_sum = 0;
		int32_t valid_values = 0;

		for(int j = 0; j < num_rows; j++) {
			for(int k = 0; k < num_cols; k++) {
				int idx = j * num_cols + k + wave_size*i;
				
				if (i == 0) {
					if (data_valid[idx])
						attr_Image_read[idx] = val[idx];
					else
						attr_Image_read[idx] = 0;
				}		
				if (data_valid[idx]) {
					hor_pos += (val[idx] * (k+1));
					ver_pos += (val[idx] * (j+1));
					total_sum += val[idx];
					valid_values++;
				}
			}
		}

		hor_pos /= total_sum;
		ver_pos /= total_sum;

		for(int i = 0; i < num_rows; i++) {
			for(int j = 0; j < num_cols; j++) {
				std_hor_pos += (i-hor_pos)*(i-hor_pos);
				std_ver_pos += (j-ver_pos)*(j-ver_pos);
			}
		}

		if (total_sum > 0) {
			std_hor_pos = pow(std_hor_pos / total_sum, 0.5);     
			std_ver_pos = pow(std_ver_pos / total_sum, 0.5);   
		}

		acq_array[array_idx].sum = total_sum;
		acq_array[array_idx].mean =  total_sum / (double)valid_values;
		acq_array[array_idx].horpos = hor_pos;
		acq_array[array_idx].verpos = ver_pos;		
		acq_array[array_idx].horposstd = std_hor_pos;
		acq_array[array_idx].verposstd = std_ver_pos;	
		acq_array[array_idx].validvalues = valid_values;

		acq_array[array_idx].bn = bn_start + i;	

		if (i == 0) {
			*attr_Sum_read =  acq_array[array_idx].sum;
			*attr_Mean_read =  acq_array[array_idx].mean;
			*attr_HorPos_read =  acq_array[array_idx].horpos;			
			*attr_VerPos_read =  acq_array[array_idx].verpos;		
			*attr_HorPosStd_read =  acq_array[array_idx].horposstd;
			*attr_VerPosStd_read =  acq_array[array_idx].verposstd;	
			*attr_ValidValues_read =  acq_array[array_idx].validvalues;					
		}

		last_bn = acq_array[array_idx].bn;	
		
		array_idx++;
		if (array_idx >= DATAVAMPIRE_MAX_SIZE)
			array_idx = 0;
		
	}
	
	free(hor_profile_bck);
	free(ver_profile_bck);
	free(hor_profile_bck_cnt);
	free(ver_profile_bck_cnt);	

}


void DataVampire::store_array_1D(Tango::DeviceData *cmd_value, int32_t bn_start, int32_t bn_end)
{
	std::vector<double> val;
	std::vector<std::string> val_string;
	int32_t num_samples = bn_end - bn_start + 1;
	int32_t roi[2];
	int32_t auto_bck_samples = *attr_AutoBackgroundSamples_read;

	
	if (roi_dim != 2) {
		Tango::Except::throw_exception (
			(const char *)"Failed to store data",
			(const char *)"Invalid ROI dimension",
			__FUNCTION__);		
	}
	
	// minimize risks of anomalies in num_cols/rows calculation 
	memcpy(roi, attr_RoiParam_read, 2 * sizeof(int32_t));

	if (*attr_EnableAcquisition_read == false) {
		Tango::Except::throw_exception (
			(const char *)"Failed to store data",
			(const char *)"Acquisition is disabled",
			__FUNCTION__);	
	}

	cmd_extract_values(cmd_value,  val, val_string);

	int num_rows, num_cols;
	if (roi_dim == 2) {
		num_cols = roi[1] - roi[0] + 1;
		num_rows = 1;
	}
	else {
		Tango::Except::throw_exception (
			(const char *)"Failed to store data",
			(const char *)"Invalid ROI setting",
			__FUNCTION__);	
	}

	int32_t wave_size = val.size() / num_samples;

	if (((num_cols * num_rows * num_samples) != val.size()) && (*attr_PostAcqRoi_read == false)) {
		Tango::Except::throw_exception (
			(const char *)"Failed to store data",
			(const char *)"Data size does not match roi setting",
			__FUNCTION__);
	}

	if (((num_cols * num_rows * num_samples) > val.size()) && (*attr_PostAcqRoi_read == true)) {
		Tango::Except::throw_exception (
			(const char *)"Failed to store data",
			(const char *)"Data size too large",
			__FUNCTION__);
	}

	vector <bool> data_valid;
	
	if (*attr_PostAcqRoi_read == false) {
		for (int i = 0; i < val.size(); i++) {
			if ((val[i] >= *attr_MinThreshold_read) && (val[i] <= *attr_MaxThreshold_read)) 
				data_valid.push_back(true);
			else
				data_valid.push_back(false);
		}
	}	
	else {
		int cnt = 0;
		for (int i = 0; i < num_samples; i++) {	
			int offset = wave_size * i;
			for (int k = roi[0]; k <= roi[1]; k++) {
				val[cnt] = val[k+offset];
				if ((val[cnt] >= *attr_MinThreshold_read) && (val[cnt] <= *attr_MaxThreshold_read)) 
					data_valid.push_back(true);
				else
					data_valid.push_back(false);
				cnt++;	
			}
		}	
		wave_size = (double)cnt / (double)num_samples;	
	}

	for (int i = 0; i < num_samples; i++) {
		double sum = 0;
		double mean = 0;
		double sum_left = 0;
		double sum_right = 0;
		double sum_top = 0;
		double sum_bottom = 0;
		double offset = 0;
		double slope = 0;
		double pos = 0;
		double std = 0;
		int cnt_left = 0, cnt_right = 0, cnt = 0;
		int32_t valid_values = 0;
		
		// calculate backround level as a slope
		if (auto_bck_samples) {
			for (int j = 0; j < auto_bck_samples; j++) {
				if (data_valid[i*wave_size+j]) {									
					double vald = val[i*wave_size+j];
					sum_left += vald;
					cnt_left++;
				}
				if (data_valid[i*wave_size+j]) {									
					double vald = val[i*wave_size+(wave_size-auto_bck_samples)+j];
					sum_right += vald;
					cnt_right++;
				}				
			}
			if (cnt_left)
				sum_left /= (double)cnt_left;
			else
				sum_left = 0;
			if (cnt_right)
				sum_right /= (double)cnt_right;
			else
				sum_right = 0;	
				
			if (auto_bck_samples > 0) {			
				slope = (sum_right - sum_left) / 
					((double) (wave_size - (double)(cnt_left+cnt_right)/2));
				offset = sum_left;
				if (offset > sum_right)
					offset = sum_right;
			}
			else { 
				slope = 0; offset = 0;
			}
	
		}

		num_cols_spectrum = num_cols;

		// get sum, mean and position
		for (int j = 0; j < num_cols; j++) {
			//printf("j=%d idx=%d\n",j,i*wave_size+j);
			double vald;
			
			if (sum_left >= sum_right)			
				vald = (val[i*wave_size+j] + slope * (double)(num_cols - j) - offset);
			else
				vald = (val[i*wave_size+j] - slope * (double)(j) - offset);
			

			// for spectrum displaying
			if (i == 0) {
				if (data_valid[i*wave_size+j]) {
					attr_Spectrum_read[j] = vald; 
				}	
				else
					attr_Spectrum_read[j] = 0;
			}
			
			if (data_valid[i*wave_size+j]) {
				sum += vald; 
				pos += vald * (double)(j + 1);
				valid_values++;
			}				
		}
		mean = sum / (double) valid_values;
		pos /= sum;

		// get std
		cnt = 0;
		for (int j = 0; j < num_cols; j++) {
			double vald = (val[i*wave_size+j] + slope * (double)j - offset);
			if (val[i*wave_size+j]) {
				std += ((double)j - pos) * ((double)j - pos); cnt++;
			}				
		}		
		if (cnt != 0)
			std = pow(std / (double) cnt, 0.5);
		else  
			std = 0;

		acq_array[array_idx].sum = sum;
		acq_array[array_idx].mean = mean;
		acq_array[array_idx].horpos = pos;
		acq_array[array_idx].horposstd = std;
		acq_array[array_idx].validvalues = valid_values;
		
		// set bunch number
		acq_array[array_idx].bn = bn_start + i;	

		if (i == 0) {
			*attr_Sum_read =  acq_array[array_idx].sum;
			*attr_Mean_read =  acq_array[array_idx].mean;
			*attr_HorPos_read =  acq_array[array_idx].horpos;					
			*attr_HorPosStd_read =  acq_array[array_idx].horposstd;
			*attr_ValidValues_read =  acq_array[array_idx].validvalues;					
		}

		last_bn = acq_array[array_idx].bn;
		
		array_idx++;
		if (array_idx >= DATAVAMPIRE_MAX_SIZE)
			array_idx = 0;		

	}
	

}


//+------------------------------------------------------------------
/**
 * method: DataVampire::store_circ_data
 * Store data into circular buffer
 */
//+------------------------------------------------------------------
void DataVampire::store_circ_data(int32_t mode, int32_t num_samples, int32_t bn_start, int32_t bn_end)
{
	std::vector<double> cmd_input_value;
	std::vector<std::string> cmd_input_value_str;
	std::vector<double> cmd_output_value;
	std::vector<std::string> cmd_output_value_str;

	cmd_input_value.push_back(mode);

	// get last mode
	if (mode == 0) {	
		cmd_input_value.push_back((double)num_samples);
		// simulate the bunch number for store_array_2D function
		bn_start = last_bn + 1;
		bn_end = last_bn + num_samples; 
		// if polling enabled let only the polling thread fill the buffer
		if (*attr_EnablePolling_read) {
			return;
		}
	}
	// get data from bn_start to bn_end
	else if (mode == 1) {
		cmd_input_value.push_back((double)bn_start);	
		cmd_input_value.push_back((double)bn_end);				
	}
	else {
		INFO_STREAM << "Not supported mode " << mode << endl;
		Tango::Except::throw_exception(
			(const char *) "Failed to store data",
			(const char *) "Not supported mode",
			__FUNCTION__, Tango::ERR);	
	}

	// data not present	
	if (bn_end < bn_start) {	
		return;
	}

	if (*attr_BackgroundSamples_read > 0) {
		cmd_input_value.push_back((double)*attr_BackgroundSamples_read);
	}	
	
	if ((*attr_PostAcqRoi_read) == false) {
		// add ROI info to params
		for (unsigned int i = 0; i < roi_dim; i++) {
			cmd_input_value.push_back((double)attr_RoiParam_read[i]);
		}	
	}	

	try {
		// prepare input
		Tango::DeviceData Din = cmd_insert_values(cmd_in_type, cmd_input_value, cmd_input_value_str);
		// get data
		Tango::DeviceData Dout = device->command_inout(command_name, Din);

		// store 1D & 2D data
		int32_t num_samples = bn_end-bn_start+1;
		if (roi_dim == 2) {
			store_array_1D(&Dout, bn_start, bn_end);
		}			
		else if (roi_dim == 4) {
			store_array_2D(&Dout, bn_start, bn_end);
		}	
	}
	catch (Tango::DevFailed &e) {
		INFO_STREAM << e.errors[0].desc << ", " << c_device_name << " " << command_name << endl;
		INFO_STREAM << e.errors[0].reason << ", " << c_device_name << " " << command_name << endl;
		Tango::Except::re_throw_exception(e,
			(const char *) "Failed to store data",
			(const char *) "Error storing data",
			__FUNCTION__, Tango::ERR);		
	}
	catch(...) {
		Tango::Except::throw_exception(
			(const char *) "Failed to store data",
			(const char *) "Generic error",
			__FUNCTION__, Tango::ERR);		
	}
	
}	


//+------------------------------------------------------------------
/**
 * method: DataVampire::get_start_index
 * get_start_index
 */
//+------------------------------------------------------------------
int32_t DataVampire::get_start_index(int32_t mode, int32_t num_samples, int32_t bn_start, int32_t bn_end)
{
	int32_t last_idx, start = -1, end = -1;
	bool bn_found = false;
	int32_t bn_samples = 0;
	
	
	// last valid sample index stored in circular buffer
	last_idx = (array_idx - 1 + DATAVAMPIRE_MAX_SIZE) % DATAVAMPIRE_MAX_SIZE;

	if (mode == 0) {
		return ((last_idx - num_samples + 1 + DATAVAMPIRE_MAX_SIZE) % DATAVAMPIRE_MAX_SIZE);
			
	}
	else if (mode == 1) {
		// first find the indexes of bn_start and bn_end
		for (int i = 0; i < DATAVAMPIRE_MAX_SIZE; i++) {
			int32_t idx = ((last_idx-i+DATAVAMPIRE_MAX_SIZE) % DATAVAMPIRE_MAX_SIZE);
			if (acq_array[idx].bn == bn_end) {
				end = idx;
			}
			if (acq_array[idx].bn == bn_start) {
				start = idx;
			}	
			if (end != -1) {
				//printf("INCREMENT bn_samples=%d\n",bn_samples);
				bn_samples++;
			}				
			if ((end != -1) && (start != -1)) {
				bn_found = true; break;		
			}	
		}	
		if ((bn_end - bn_start + 1) != bn_samples) {
			Tango::Except::throw_exception(
				(const char *) "Failed to find buffer indexes",
				(const char *) "Data missing",
				__FUNCTION__, Tango::ERR);		
		}
			
		if (bn_found) {
			return start;
		}
		else {
			Tango::Except::throw_exception(
				(const char *) "Failed to find buffer indexes",
				(const char *) "Can't find start/end bunch numbers",
				__FUNCTION__, Tango::ERR);					
		}
	}
	else {
		Tango::Except::throw_exception(
			(const char *) "Failed to find buffer indexes",
			(const char *) "Buffer mode not supported",
			__FUNCTION__, Tango::ERR);	
	}	

}


//+------------------------------------------------------------------
/**
 * method: DataVampire::get_param_in
 * Get input parameters
 */
//+------------------------------------------------------------------
void DataVampire::get_param_in(const Tango::DevVarLongArray *argin, int32_t *mode, int32_t *num_samples, int32_t *bn_start, int32_t *bn_end)
{
	int32_t arglen = argin->length();

	*mode = ((*argin)[0] % 10) & 0x1; // 20->0 21->1

	if (arglen == 2) {
		*num_samples = (*argin)[1];
		*bn_start = 0;	
		*bn_end = 0;
		
		if (*mode != 0) {
			Tango::Except::throw_exception(
				(const char *) "Failed to parse input parameter",
				(const char *) "Invalid mode (0)",
				__FUNCTION__, Tango::ERR);			
		}
		else {
			if ((*num_samples < 1) || (*num_samples > DATAVAMPIRE_MAX_SIZE)) {
				Tango::Except::throw_exception(
					(const char *) "Failed to parse input parameter",
					(const char *) "Invalid number of samples",
					__FUNCTION__, Tango::ERR);			
			}
		}
	}
	else if (arglen == 3) {
		*bn_start = (*argin)[1];	
		*bn_end = (*argin)[2];
		*num_samples = *bn_end - *bn_start + 1;
	
		if (*mode != 1) {
			Tango::Except::throw_exception(
				(const char *) "Failed to parse input parameter",
				(const char *) "Invalid mode (1)",
				__FUNCTION__, Tango::ERR);			
		}
		else {
			if ((*num_samples < 1) || (*num_samples > DATAVAMPIRE_MAX_SIZE)) {
				Tango::Except::throw_exception(
					(const char *) "Failed to parse input parameter",
					(const char *) "Invalid number of samples",
					__FUNCTION__, Tango::ERR);			
			}
		}	
	}
	else {
		Tango::Except::throw_exception(
			(const char *) "Failed to parse input parameter",
			(const char *) "Mode not recognized (0/1)",
			__FUNCTION__, Tango::ERR);	
	}

}


/*----- PROTECTED REGION END -----*/	//	DataVampire::namespace_ending
} //	namespace
